HORIZON_HOST = "horizons.jpl.nasa.gov"
HORIZON_PORT = 6775
HORIZON_PROMPT = "Horizons>"
HORIZON_DEFAULT_PROMPT = ":"
HORIZON_QUERY_PROMPT = "<cr>:"
HORIZON_OPTION_PROMPT = " --> "
HORIZON_CARTESIAN_PROMPT = "? : "
HORIZON_MISC_PROMPT = "] : "
DEBUG = False
